﻿Public Class AccountController
    Inherits System.Web.Mvc.Controller

    Function Login() As ActionResult
        Return View()
    End Function

    <HttpPost()>
    Function Login(ByVal viewmodel As ViewModelLogin, ByVal collection As FormCollection) As ActionResult
        Try
            If collection("button") = "ログイン" Then
                Dim service As New LoginService

                If Not ModelState.IsValid Then
                    Return View(viewmodel)
                End If

                Dim ExistsUser As Boolean = service.IsMstUser(viewmodel)
                viewmodel.IP_ADDRESS = System.Web.HttpContext.Current.Request.UserHostAddress
                viewmodel.SESSION_ID = Session.SessionID

                If ExistsUser = True Then
                    viewmodel = service.GetMstUser(viewmodel)

                    ' ロックされているときメッセージを表示する
                    If viewmodel.LOCK = True Then
                        ModelState.AddModelError("", "オペレータ(" & viewmodel.OPERATOR_ID & ")はロックされています。管理者までご連絡ください")

                        If Not ModelState.IsValid Then
                            Return View()
                        End If
                    End If

                    ' 無効化されているとき
                    If viewmodel.DISABLED = True Then
                        ModelState.AddModelError("", "オペレータ(" & collection.Item("operator_id") & ")は無効化されています。管理者までご連絡ください")

                        If Not ModelState.IsValid Then
                            Return View()
                        End If
                    End If

                    Dim encryptPassword As String = Crypt.EncryptSHA256(viewmodel.PASSWORD)

                    If viewmodel.ENCRYPTED_PASSWORD = encryptPassword Then
                        ' 初期化されたパスワードのとき
                        If viewmodel.INITIAL_PASSWORD = True Then
                            ModelState.AddModelError("", "初期化されたパスワードです。パスワードを更新してください")

                            If Not ModelState.IsValid Then
                                Return View()
                            End If
                        End If

                        ' パスワードの有効期限が過ぎたとき
                        If DateTime.Parse(viewmodel.LAST_PASSWORD_CHANGE).AddDays(120) < DateTime.Now Then
                            ModelState.AddModelError("", "パスワードの有効期限が過ぎています。パスワードを更新してください")

                            If Not ModelState.IsValid Then
                                Return View()
                            End If
                        End If


                        service.LoginSuccessful(viewmodel)
                    Else
                        service.LoginFailed(viewmodel)

                        ' ログインに失敗したことを表示
                        ModelState.AddModelError("", "ログインに失敗しました")

                        Return View()
                    End If
                Else

                    ' ログインに失敗したことを表示
                    ModelState.AddModelError("", "ログインに失敗しました")

                    Return View()
                End If

                FormsAuthentication.SetAuthCookie(viewmodel.OPERATOR_ID, False)

                Dim accInfo As AccountInfo = New AccountInfo
                accInfo.IsLogin = True
                accInfo.OPERATOR_ID = viewmodel.OPERATOR_ID
                accInfo.OPERATOR_NAME = viewmodel.OPERATOR_NAME
                accInfo.LAST_PASSWORD_CHANGE = viewmodel.LAST_PASSWORD_CHANGE
                accInfo.LAST_LOGIN_DATE = viewmodel.LAST_LOGIN_DATE
                accInfo.LAST_LOGIN_DATE2 = viewmodel.LAST_LOGIN_DATE2
                accInfo.ROLE = viewmodel.ROLE

                Session("AccountInfo") = accInfo
                Return RedirectToAction("Index", "Home")
            Else
                FormsAuthentication.SetAuthCookie(viewmodel.OPERATOR_ID, False)

                Return RedirectToAction("ChangePassword", "Account")
            End If
        Catch ex As Exception
            ModelState.AddModelError("", "ログイン中にエラーが発生しました。")


            Return View(viewmodel)
        End Try



    End Function

    Function Logout() As ActionResult
        Session.Clear()

        Return RedirectToAction("Login", "Account")
    End Function

    Public Function ChangePassword() As ActionResult
        Dim viewmodel As ViewModelChangePassword = TryCast(Session("PasswordChangeData"), ViewModelChangePassword)

        If viewmodel Is Nothing Then
            viewmodel = New ViewModelChangePassword
        End If

        Return View(viewmodel)
    End Function

    <HttpPost()> _
    Public Function ChangePassword(ByVal viewmodel As ViewModelChangePassword, ByVal collection As FormCollection) As ActionResult
        If collection("button") = "確認" Then
            viewmodel.MASKED_PASSWORD = Util.Mask(viewmodel.NEW_PASSWORD)

            If ModelState.IsValid = True Then
                Session("PasswordChangeData") = viewmodel
                Return RedirectToAction("ChangePasswordConfirm", "Account")
            Else
                Return View(viewmodel)
            End If

        Else
            Session("PasswordChangeData") = Nothing
            Return RedirectToAction("Login", "Account")
        End If
    End Function

    Public Function ChangePasswordConfirm() As ActionResult
        Dim viewmodel As ViewModelChangePassword = TryCast(Session("PasswordChangeData"), ViewModelChangePassword)

        Return View(viewmodel)
    End Function

    <HttpPost()> _
    Public Function ChangePasswordConfirm(ByVal viewmodel As ViewModelChangePassword, ByVal collection As FormCollection) As ActionResult
        Dim data As ViewModelChangePassword = TryCast(Session("PasswordChangeData"), ViewModelChangePassword)

        If collection("button") = "保存" Then

            Dim service As New LoginService
            Try
                service.SaveChangePassword(data)
            Catch ex As Exception
                ' メッセージボックスを表示
                Session("MessageBoxTitle") = "エラーが発生"
                Session("MessageBoxMessage") = "パスワード変更中にエラーが発生しました"

                Session("PasswordChangeData") = Nothing

                Return RedirectToAction("MessageBox", "Account")
            End Try

            ' メッセージボックスを表示
            Session("MessageBoxTitle") = "パスワード変更完了"
            Session("MessageBoxMessage") = "パスワード変更が完了しました"

            Session("PasswordChangeData") = Nothing

            Return RedirectToAction("MessageBox", "Account")
        Else
            Return RedirectToAction("ChangePassword", "Account")
        End If
    End Function

    Function MessageBox() As ActionResult
        ViewData("MessagaBoxTitle") = Session("MessageBoxTitle")
        ViewData("MessageBoxMessage") = Session("MessageBoxMessage")

        Return View()
    End Function

    <HttpPost()>
    Function MessageBox(ByVal collection As FormCollection) As ActionResult
        Session("MessageBoxTitle") = Nothing
        Session("MessageBoxMessage") = Nothing

        Return RedirectToAction("Login", "Account")
    End Function


End Class