﻿Public Class HomeController
    Inherits System.Web.Mvc.Controller

    Function Index() As ActionResult
        Return View()
    End Function

    Function ReservationRoom() As ActionResult
        Dim acc As AccountInfo = TryCast(Session("AccountInfo"), AccountInfo)

        If acc Is Nothing OrElse Not acc.ROLE("Reservation") Then
            Return RedirectToAction("Maintenance", "Home")
        End If

        Dim viewmodel As ViewModelReservationRoom = TryCast(Session("ReservationData"), ViewModelReservationRoom)

        If viewmodel Is Nothing Then
            viewmodel = New ViewModelReservationRoom
        End If

        Dim service As New BuildingService

        viewmodel.BUILDING_COMBOBOX = service.SetBuildingList
        viewmodel.BUILDING = viewmodel.BUILDING_COMBOBOX.First.Value

        viewmodel.ROOM_COMBOBOX = service.SetRoomList(viewmodel.BUILDING)
        viewmodel.ROOM = viewmodel.ROOM_COMBOBOX.First.Value

        viewmodel.COMMENT = service.SetRoomComment(viewmodel.ROOM)

        Dim minTime As String = service.SetRoomStartTime(viewmodel.ROOM)
        Dim maxTime As String = service.SetRoomEndTime(viewmodel.ROOM)

        viewmodel.START_TIME_COMBOBOX = service.SetReserveStartTime(minTime, maxTime)
        viewmodel.END_TIME_COMBOBOX = service.SetReserveEndTime(minTime, maxTime)


        Return View(viewmodel)
    End Function

    <HttpPost()>
    Function ReservationRoom(ByVal viewmodel As ViewModelReservationRoom, ByVal collection As FormCollection) As ActionResult
        Dim acc As AccountInfo = TryCast(Session("AccountInfo"), AccountInfo)

        If acc Is Nothing OrElse Not acc.ROLE("Reservation") Then
            Return RedirectToAction("Maintenance", "Home")
        End If
        Dim service As New BuildingService
        If collection.Item("button") = "確認" Then    ' 確認ボタンが押されてとき
            If ModelState.IsValid = False Then
                Return View(viewmodel)
            End If

            Dim AlreadyReserve As Boolean = service.AlreadyReserve(viewmodel)

            If AlreadyReserve = True Then
                ModelState.AddModelError("", "指定の時間帯は予約済みです。")
                viewmodel.BUILDING_COMBOBOX = service.SetBuildingList
                viewmodel.ROOM_COMBOBOX = service.SetRoomList(viewmodel.BUILDING)
                viewmodel.COMMENT = service.SetRoomComment(viewmodel.ROOM)
                Dim minTime As String = service.SetRoomStartTime(viewmodel.ROOM)
                Dim maxTime As String = service.SetRoomEndTime(viewmodel.ROOM)
                viewmodel.START_TIME_COMBOBOX = service.SetReserveStartTime(minTime, maxTime)
                viewmodel.END_TIME_COMBOBOX = service.SetReserveEndTime(minTime, maxTime)
                Return View(viewmodel)
            End If


            viewmodel.BUILDING_NAME = service.SetBuildingName(viewmodel.BUILDING)
            viewmodel.ROOM_NAME = service.SetRoomName(viewmodel.ROOM)
            viewmodel.COMMENT = service.SetRoomComment(viewmodel.ROOM)

            Session("ReservationData") = viewmodel

            Return RedirectToAction("ReservationRoomConfirm", "Home")
        Else


            viewmodel.BUILDING_COMBOBOX = service.SetBuildingList
            viewmodel.BUILDING = viewmodel.BUILDING

            viewmodel.ROOM_COMBOBOX = service.SetRoomList(viewmodel.BUILDING)
            viewmodel.ROOM = viewmodel.ROOM

            viewmodel.COMMENT = service.SetRoomComment(viewmodel.ROOM)

            Dim minTime As String = service.SetRoomStartTime(viewmodel.ROOM)
            Dim maxTime As String = service.SetRoomEndTime(viewmodel.ROOM)

            viewmodel.START_TIME_COMBOBOX = service.SetReserveStartTime(minTime, maxTime)
            viewmodel.END_TIME_COMBOBOX = service.SetReserveEndTime(minTime, maxTime)


            Return View(viewmodel)
        End If
    End Function

    Function ReservationRoomConfirm() As ActionResult
        Dim acc As AccountInfo = TryCast(Session("AccountInfo"), AccountInfo)

        If acc Is Nothing OrElse Not acc.ROLE("Reservation") Then
            Return RedirectToAction("Maintenance", "Home")
        End If
        Dim viewmodel As ViewModelReservationRoom = TryCast(Session("ReservationData"), ViewModelReservationRoom)

        ' セッションがないときはエラー
        If viewmodel Is Nothing Then
            Return RedirectToAction("Maintenance", "Home")
        End If

        Return View(viewmodel)

    End Function

    <HttpPost()>
    Function ReservationRoomConfirm(ByVal collection As FormCollection) As ActionResult
        Dim acc As AccountInfo = TryCast(Session("AccountInfo"), AccountInfo)

        If acc Is Nothing OrElse Not acc.ROLE("Reservation") Then
            Return RedirectToAction("Maintenance", "Home")
        End If
        Dim viewmodel As ViewModelReservationRoom = TryCast(Session("ReservationData"), ViewModelReservationRoom)
        Dim service As New BuildingService
        ' セッションがないときはエラー
        If viewmodel Is Nothing Then
            Return RedirectToAction("Maintenance", "Home")
        End If


        If collection.Item("button") = "保存" Then ' 保存ボタンが押されたとき
            Dim AlreadyReserve As Boolean = service.AlreadyReserve(viewmodel)

            If AlreadyReserve = True Then
                ModelState.AddModelError("", "指定の時間帯は予約済みです。")
                Return View(viewmodel)
            End If

            Try
                service.SaveReservation(viewmodel)
            Catch ex As Exception
                Return RedirectToAction("Maintenance", "Home")
            End Try

            ' セッションをクリア
            Session("ReservationData") = Nothing
            ' メッセージボックスの表示
            Session("MessageBoxTitle") = "会議室予約完了"
            Session("MessageBoxMessage") = "会議室の予約が完了しました"
            Return RedirectToAction("MessageBox", "Home")

        Else
            ' 予約画面へ戻る
            Return RedirectToAction("ReservationRoom", "Home")
        End If

    End Function

    Function ReservationConfirm() As ActionResult
        Dim acc As AccountInfo = TryCast(Session("AccountInfo"), AccountInfo)

        If acc Is Nothing OrElse Not acc.ROLE("Reservation") Then
            Return RedirectToAction("Maintenance", "Home")
        End If
        Dim viewmodel = New ViewModelReservationConfirm

        Dim service As New BuildingService

        viewmodel.BUILDING_COMBOBOX = service.SetBuildingList
        viewmodel.BUILDING = viewmodel.BUILDING_COMBOBOX.First.Value


        Return View(viewmodel)
    End Function

    <HttpPost()>
    Function ReservationConfirm(ByVal viewmodel As ViewModelReservationConfirm, ByVal collection As FormCollection) As ActionResult
        Dim acc As AccountInfo = TryCast(Session("AccountInfo"), AccountInfo)

        If acc Is Nothing OrElse Not acc.ROLE("Reservation") Then
            Return RedirectToAction("Maintenance", "Home")
        End If
        Dim service As New BuildingService
        viewmodel.BUILDING_COMBOBOX = service.SetBuildingList
        viewmodel.RESERVATION = service.GetReservation(viewmodel)

        Return View(viewmodel)
    End Function

    Function EditReservation(ByVal id As String, ByVal d As String) As ActionResult
        Dim acc As AccountInfo = TryCast(Session("AccountInfo"), AccountInfo)

        If acc Is Nothing OrElse Not acc.ROLE("Reservation") Then
            Return RedirectToAction("Maintenance", "Home")
        End If
        Dim viewmodel As New ViewModelEditReservation
        Dim service As New BuildingService
        viewmodel.RESERVATION = service.GetReservationList(id, d)

        If viewmodel.RESERVATION.Count = 0 Then
            Return RedirectToAction("Maintenance", "Home")
        End If

        Return View(viewmodel)
    End Function

    <HttpPost()>
    Function EditReservation(ByVal collection As FormCollection) As ActionResult
        Dim acc As AccountInfo = TryCast(Session("AccountInfo"), AccountInfo)

        If acc Is Nothing OrElse Not acc.ROLE("Reservation") Then
            Return RedirectToAction("Maintenance", "Home")
        End If
        If collection.Item("button") = "戻る" Then
            Return RedirectToAction("ReservationConfirm", "Home")
        Else

            Dim DeleteButtonNo As Integer
            ' 押された削除ボタンを判定
            For Each keys In collection.Keys
                If keys.ToString.StartsWith("remove_") Then
                    DeleteButtonNo = CInt(keys.ToString.Replace("remove_", ""))
                End If
            Next

            Dim service As New BuildingService

            Try
                service.CancelReservation(DeleteButtonNo)
            Catch ex As Exception
                Return RedirectToAction("Maintenance", "Home")
            End Try

            ' メッセージボックスの表示
            Session("MessageBoxTitle") = "予約取消"
            Session("MessageBoxMessage") = "予約を取り消しました"


            Return RedirectToAction("MessageBox", "Home")

        End If
    End Function

    Function EditMeetingRoom() As ActionResult
        Dim acc As AccountInfo = TryCast(Session("AccountInfo"), AccountInfo)

        If acc Is Nothing OrElse Not acc.ROLE("Reservation") Then
            Return RedirectToAction("Maintenance", "Home")
        End If
        Return View()
    End Function

    Function ListBuilding(ByVal page As Integer?) As ActionResult
        Dim acc As AccountInfo = TryCast(Session("AccountInfo"), AccountInfo)

        If acc Is Nothing OrElse Not acc.ROLE("Reservation Manager") Then
            Return RedirectToAction("Maintenance", "Home")
        End If
        Dim viewmodel As New ViewModelBuilding
        Dim service As New BuildingService
        If Not page Is Nothing Then
            Try
                viewmodel = service.GetBuildingList(viewmodel, page - 1)
            Catch ex As Exception
                Return RedirectToAction("Maintenance", "Home")
            End Try
        End If
        Return View(viewmodel)
    End Function

    <HttpPost()>
    Function ListBuilding(ByVal viewmodel As ViewModelBuilding, ByVal collection As FormCollection) As ActionResult
        Dim acc As AccountInfo = TryCast(Session("AccountInfo"), AccountInfo)

        If acc Is Nothing OrElse Not acc.ROLE("Reservation Manager") Then
            Return RedirectToAction("Maintenance", "Home")
        End If
        If collection("button") = "検索" Then
            Dim service As New BuildingService

            Try
                viewmodel = service.GetBuildingList(viewmodel, 0)
            Catch ex As Exception
                Return RedirectToAction("Maintenance", "Home")
            End Try

            Return View(viewmodel)
        Else
            Session("Screen") = "NewBuilding"
            Return RedirectToAction("NewBuilding", "Home")
        End If
    End Function

    Function NewBuilding() As ActionResult
        Dim acc As AccountInfo = TryCast(Session("AccountInfo"), AccountInfo)

        If acc Is Nothing OrElse Not acc.ROLE("Reservation Manager") Then
            Return RedirectToAction("Maintenance", "Home")
        End If

        If Session("Screen") <> "NewBuilding" Then
            Return RedirectToAction("Maintenance", "Home")
        End If

        Dim viewmodel As ViewModelNewBuilding = TryCast(Session("NewBuildingData"), ViewModelNewBuilding)

        If viewmodel Is Nothing Then
            viewmodel = New ViewModelNewBuilding
        End If

        Return View(viewmodel)

    End Function

    <HttpPost()>
    Function NewBuilding(ByVal viewmodel As ViewModelNewBuilding, ByVal collection As FormCollection) As ActionResult
        Dim acc As AccountInfo = TryCast(Session("AccountInfo"), AccountInfo)

        If acc Is Nothing OrElse Not acc.ROLE("Reservation Manager") Then
            Return RedirectToAction("Maintenance", "Home")
        End If

        If Session("Screen") <> "NewBuilding" Then
            Return RedirectToAction("Maintenance", "Home")
        End If


        If collection("button") = "戻る" Then ' 戻るボタンを押したとき
            Session("Screen") = Nothing
            Return RedirectToAction("ListBuilding", "Home")
        Else

            Session("NewBuildingData") = viewmodel

            If ModelState.IsValid = True Then
                Session("Screen") = "NewBuildingConfirm"
                Return RedirectToAction("NewBuildingConfirm", "Home")
            Else
                Return View(viewmodel)
            End If
        End If
    End Function

    Function NewBuildingConfirm() As ActionResult
        Dim acc As AccountInfo = TryCast(Session("AccountInfo"), AccountInfo)

        If acc Is Nothing OrElse Not acc.ROLE("Reservation Manager") Then
            Return RedirectToAction("Maintenance", "Home")
        End If

        If Session("Screen") <> "NewBuildingConfirm" Then
            Return RedirectToAction("Maintenance", "Home")
        End If


        Dim viewmodel As ViewModelNewBuilding = TryCast(Session("NewBuildingData"), ViewModelNewBuilding)

        Return View(viewmodel)
    End Function

    <HttpPost()>
    Function NewBuildingConfirm(ByVal collection As FormCollection) As ActionResult
        Dim acc As AccountInfo = TryCast(Session("AccountInfo"), AccountInfo)

        If acc Is Nothing OrElse Not acc.ROLE("Reservation Manager") Then
            Return RedirectToAction("Maintenance", "Home")
        End If

        If Session("Screen") <> "NewBuildingConfirm" Then
            Return RedirectToAction("Maintenance", "Home")
        End If

        If collection("button") = "戻る" Then
            Session("Screen") = "NewBuilding"
            Return RedirectToAction("NewBuilding", "Home")
        Else
            Dim viewmodel As ViewModelNewBuilding = TryCast(Session("NewBuildingData"), ViewModelNewBuilding)

            Dim service As New BuildingService

            Try
                service.SaveNewBuilding(viewmodel)
            Catch ex As Exception
                Return RedirectToAction("Maintenance", "Home")
            End Try


            ' メッセージボックスの表示
            Session("MessageBoxTitle") = "ビルの登録完了"
            Session("MessageBoxMessage") = "ビルの登録が完了しました"

            Session("NewBuildingData") = Nothing
            Session("Screen") = Nothing

            Return RedirectToAction("MessageBox", "Home")

        End If

    End Function

    Function EditBuilding(ByVal id As String) As ActionResult
        Dim acc As AccountInfo = TryCast(Session("AccountInfo"), AccountInfo)

        If acc Is Nothing OrElse Not acc.ROLE("Reservation Manager") Then
            Return RedirectToAction("Maintenance", "Home")
        End If
        Dim viewmodel As ViewModelEditBuilding = TryCast(Session("EditBuildingData"), ViewModelEditBuilding)
        Dim service As New BuildingService

        Try
            If viewmodel Is Nothing Then
                viewmodel = New ViewModelEditBuilding
                viewmodel.SEQ_ID = id
                viewmodel = service.GetEditBuildingData(viewmodel)
            End If
        Catch ex As Exception
            Return RedirectToAction("Maintenance", "Home")
        End Try

        Return View(viewmodel)
    End Function

    <HttpPost()>
    Function EditBuilding(ByVal id As String, ByVal viewmodel As ViewModelEditBuilding, ByVal collection As FormCollection) As ActionResult
        Dim acc As AccountInfo = TryCast(Session("AccountInfo"), AccountInfo)

        If acc Is Nothing OrElse Not acc.ROLE("Reservation Manager") Then
            Return RedirectToAction("Maintenance", "Home")
        End If
        If collection("button") = "戻る" Then ' 戻るボタンを押したとき
            Session("EditBuildingData") = Nothing
            Session("Screen") = Nothing
            Return RedirectToAction("ListBuilding", "Home")
        Else

            viewmodel.SEQ_ID = id
            Session("EditBuildingData") = viewmodel

            If ModelState.IsValid = True Then
                Session("Screen") = "EditBuildingConfirm"
                Return RedirectToAction("EditBuildingConfirm", "Home")
            Else
                Return View(viewmodel)
            End If
        End If
    End Function

    Function EditBuildingConfirm() As ActionResult
        Dim acc As AccountInfo = TryCast(Session("AccountInfo"), AccountInfo)

        If acc Is Nothing OrElse Not acc.ROLE("Reservation Manager") Then
            Return RedirectToAction("Maintenance", "Home")
        End If

        If Session("Screen") <> "EditBuildingConfirm" Then
            Return RedirectToAction("Maintenance", "Home")
        End If


        Dim viewmodel As ViewModelEditBuilding = TryCast(Session("EditBuildingData"), ViewModelEditBuilding)

        Return View(viewmodel)
    End Function

    <HttpPost()>
    Function EditBuildingConfirm(ByVal collection As FormCollection) As ActionResult
        Dim acc As AccountInfo = TryCast(Session("AccountInfo"), AccountInfo)

        If acc Is Nothing OrElse Not acc.ROLE("Reservation Manager") Then
            Return RedirectToAction("Maintenance", "Home")
        End If

        If Session("Screen") <> "EditBuildingConfirm" Then
            Return RedirectToAction("Maintenance", "Home")
        End If

        Dim viewmodel As ViewModelEditBuilding = TryCast(Session("EditBuildingData"), ViewModelEditBuilding)

        If collection("button") = "戻る" Then ' 戻るボタンを押したとき
            Session("Screen") = "EditBuilding"
            Return RedirectToAction("EditBuilding/" & viewmodel.SEQ_ID, "Home")
        Else ' 保存ボタンを押したとき
            Dim service As New BuildingService
            Try
                service.SaveEditBuilding(viewmodel)
            Catch ex As Exception
                Return RedirectToAction("Maintenance", "Home")
            End Try
            ' メッセージボックスの表示
            Session("MessageBoxTitle") = "ビルの更新完了"
            Session("MessageBoxMessage") = "ビルの更新が完了しました"

            Session("EditBuildingData") = Nothing
            Session("Screen") = Nothing

            Return RedirectToAction("MessageBox", "Home")
        End If
    End Function

    Function ListRoom(ByVal page As Integer?) As ActionResult
        Dim acc As AccountInfo = TryCast(Session("AccountInfo"), AccountInfo)

        If acc Is Nothing OrElse Not acc.ROLE("Reservation Manager") Then
            Return RedirectToAction("Maintenance", "Home")
        End If
        Dim viewmodel As New ViewModelRoom
        Dim service As New BuildingService
        Try
            viewmodel.BUILDING_COMBOBOX = service.SetBuildingList()
        Catch ex As Exception
            Return RedirectToAction("Maintenance", "Home")
        End Try
        If Not page Is Nothing Then
            Try
                viewmodel = service.GetRoomList(viewmodel, page - 1)
            Catch ex As Exception
                Return RedirectToAction("Maintenance", "Home")
            End Try
        End If
        Return View(viewmodel)
    End Function

    <HttpPost()>
    Function ListRoom(ByVal viewmodel As ViewModelRoom, ByVal collection As FormCollection) As ActionResult
        Dim acc As AccountInfo = TryCast(Session("AccountInfo"), AccountInfo)

        If acc Is Nothing OrElse Not acc.ROLE("Reservation Manager") Then
            Return RedirectToAction("Maintenance", "Home")
        End If
        If collection("button") = "検索" Then
            Dim service As New BuildingService
            Try
                viewmodel.BUILDING_COMBOBOX = service.SetBuildingList()
                viewmodel = service.GetRoomList(viewmodel, 0)
            Catch ex As Exception
                Return RedirectToAction("Maintenance", "Home")
            End Try

            Return View(viewmodel)
        Else
            Session("Screen") = "NewRoom"
            Return RedirectToAction("NewRoom", "Home")
        End If
    End Function

    Function NewRoom() As ActionResult
        Dim acc As AccountInfo = TryCast(Session("AccountInfo"), AccountInfo)

        If acc Is Nothing OrElse Not acc.ROLE("Reservation Manager") Then
            Return RedirectToAction("Maintenance", "Home")
        End If

        If Session("Screen") <> "NewRoom" Then
            Return RedirectToAction("Maintenance", "Home")
        End If

        Dim viewmodel As ViewModelNewRoom = TryCast(Session("NewRoomData"), ViewModelNewRoom)
        Dim service As New BuildingService

        If viewmodel Is Nothing Then

            viewmodel = New ViewModelNewRoom
        End If
        Try
            viewmodel.BUILDING_COMBOBOX = service.SetBuildingList
            viewmodel.START_TIME_COMBOBOX = service.SetStartEndTime
            viewmodel.END_TIME_COMBOBOX = service.SetStartEndTime
        Catch ex As Exception
            Return RedirectToAction("Maintenance", "Home")
        End Try


        Return View(viewmodel)

    End Function

    <HttpPost()>
    Function NewRoom(ByVal viewmodel As ViewModelNewRoom, ByVal collection As FormCollection) As ActionResult
        Dim acc As AccountInfo = TryCast(Session("AccountInfo"), AccountInfo)

        If acc Is Nothing OrElse Not acc.ROLE("Reservation Manager") Then
            Return RedirectToAction("Maintenance", "Home")
        End If

        If Session("Screen") <> "NewRoom" Then
            Return RedirectToAction("Maintenance", "Home")
        End If


        If collection("button") = "戻る" Then ' 戻るボタンを押したとき
            Session("Screen") = Nothing
            Return RedirectToAction("ListRoom", "Home")
        Else
            Dim service As New BuildingService
            Try
                viewmodel.BUILDING_NAME = service.SetBuildingName(viewmodel.BUILDING)
            Catch ex As Exception
                Return RedirectToAction("Maintenance", "Home")
            End Try

            Session("NewRoomData") = viewmodel

            If ModelState.IsValid = True Then
                Session("Screen") = "NewRoomConfirm"
                Return RedirectToAction("NewRoomConfirm", "Home")
            Else
                Return View(viewmodel)
            End If
        End If
    End Function

    Function NewRoomConfirm() As ActionResult
        Dim acc As AccountInfo = TryCast(Session("AccountInfo"), AccountInfo)

        If acc Is Nothing OrElse Not acc.ROLE("Reservation Manager") Then
            Return RedirectToAction("Maintenance", "Home")
        End If

        If Session("Screen") <> "NewRoomConfirm" Then
            Return RedirectToAction("Maintenance", "Home")
        End If


        Dim viewmodel As ViewModelNewRoom = TryCast(Session("NewRoomData"), ViewModelNewRoom)

        Return View(viewmodel)
    End Function

    <HttpPost()>
    Function NewRoomConfirm(ByVal collection As FormCollection) As ActionResult
        Dim acc As AccountInfo = TryCast(Session("AccountInfo"), AccountInfo)

        If acc Is Nothing OrElse Not acc.ROLE("Reservation Manager") Then
            Return RedirectToAction("Maintenance", "Home")
        End If

        If Session("Screen") <> "NewRoomConfirm" Then
            Return RedirectToAction("Maintenance", "Home")
        End If

        If collection("button") = "戻る" Then
            Session("Screen") = "NewRoom"
            Return RedirectToAction("NewRoom", "Home")
        Else
            Dim viewmodel As ViewModelNewRoom = TryCast(Session("NewRoomData"), ViewModelNewRoom)

            Dim service As New BuildingService

            Try
                service.SaveNewRoom(viewmodel)
            Catch ex As Exception
                Return RedirectToAction("Maintenance", "Home")
            End Try


            ' メッセージボックスの表示
            Session("MessageBoxTitle") = "会議室の登録完了"
            Session("MessageBoxMessage") = "会議室の登録が完了しました"

            Session("NewRoomData") = Nothing
            Session("Screen") = Nothing

            Return RedirectToAction("MessageBox", "Home")

        End If

    End Function

    Function EditRoom(ByVal id As String) As ActionResult
        Dim acc As AccountInfo = TryCast(Session("AccountInfo"), AccountInfo)

        If acc Is Nothing OrElse Not acc.ROLE("Reservation Manager") Then
            Return RedirectToAction("Maintenance", "Home")
        End If
        Dim viewmodel As ViewModelEditRoom = TryCast(Session("EditRoomData"), ViewModelEditRoom)
        Dim service As New BuildingService

        Try
            If viewmodel Is Nothing Then
                viewmodel = New ViewModelEditRoom
                viewmodel.SEQ_ID = id
                viewmodel = service.GetEditRoomData(viewmodel)
            End If
        Catch ex As Exception
            Return RedirectToAction("Maintenance", "Home")
        End Try

        Try
            viewmodel.BUILDING_COMBOBOX = service.SetBuildingList
            viewmodel.START_TIME_COMBOBOX = service.SetStartEndTime
            viewmodel.END_TIME_COMBOBOX = service.SetStartEndTime
        Catch ex As Exception
            Return RedirectToAction("Maintenance", "Home")
        End Try



        Return View(viewmodel)
    End Function

    <HttpPost()>
    Function EditRoom(ByVal id As String, ByVal viewmodel As ViewModelEditRoom, ByVal collection As FormCollection) As ActionResult
        Dim acc As AccountInfo = TryCast(Session("AccountInfo"), AccountInfo)

        If acc Is Nothing OrElse Not acc.ROLE("Reservation Manager") Then
            Return RedirectToAction("Maintenance", "Home")
        End If
        If collection("button") = "戻る" Then ' 戻るボタンを押したとき
            Session("EditRoomData") = Nothing
            Session("Screen") = Nothing
            Return RedirectToAction("ListRoom", "Home")
        Else
            Dim service As New BuildingService

            viewmodel.SEQ_ID = id
            viewmodel.BUILDING_NAME = service.SetBuildingName(viewmodel.BUILDING)
            Session("EditRoomData") = viewmodel

            If ModelState.IsValid = True Then
                Session("Screen") = "EditRoomConfirm"
                Return RedirectToAction("EditRoomConfirm", "Home")
            Else
                Return View(viewmodel)
            End If
        End If
    End Function

    Function EditRoomConfirm() As ActionResult
        Dim acc As AccountInfo = TryCast(Session("AccountInfo"), AccountInfo)

        If acc Is Nothing OrElse Not acc.ROLE("Reservation Manager") Then
            Return RedirectToAction("Maintenance", "Home")
        End If

        If Session("Screen") <> "EditRoomConfirm" Then
            Return RedirectToAction("Maintenance", "Home")
        End If


        Dim viewmodel As ViewModelEditRoom = TryCast(Session("EditRoomData"), ViewModelEditRoom)

        Return View(viewmodel)
    End Function

    <HttpPost()>
    Function EditRoomConfirm(ByVal collection As FormCollection) As ActionResult
        Dim acc As AccountInfo = TryCast(Session("AccountInfo"), AccountInfo)

        If acc Is Nothing OrElse Not acc.ROLE("Reservation Manager") Then
            Return RedirectToAction("Maintenance", "Home")
        End If

        If Session("Screen") <> "EditRoomConfirm" Then
            Return RedirectToAction("Maintenance", "Home")
        End If

        Dim viewmodel As ViewModelEditRoom = TryCast(Session("EditRoomData"), ViewModelEditRoom)

        If collection("button") = "戻る" Then ' 戻るボタンを押したとき
            Session("Screen") = "EditRoom"
            Return RedirectToAction("EditRoom/" & viewmodel.SEQ_ID, "Home")
        Else ' 保存ボタンを押したとき
            Dim service As New BuildingService
            Try
                service.SaveEditRoom(viewmodel)
            Catch ex As Exception
                Return RedirectToAction("Maintenance", "Home")
            End Try
            ' メッセージボックスの表示
            Session("MessageBoxTitle") = "会議室の更新完了"
            Session("MessageBoxMessage") = "会議室の更新が完了しました"

            Session("EditBuildingData") = Nothing
            Session("Screen") = Nothing

            Return RedirectToAction("MessageBox", "Home")
        End If
    End Function


    Function MessageBox() As ActionResult
        ViewData("MessagaBoxTitle") = Session("MessageBoxTitle")
        ViewData("MessageBoxMessage") = Session("MessageBoxMessage")

        Return View()
    End Function

    <HttpPost()>
    Function MessageBox(ByVal collection As FormCollection) As ActionResult
        Session("MessageBoxTitle") = Nothing
        Session("MessageBoxMessage") = Nothing

        Return RedirectToAction("Index", "Home")
    End Function

    Function Maintenance() As ActionResult
        Session.Clear()

        Return View()
    End Function
End Class
