﻿Imports System.Transactions

Public Class BuildingService
    Public Function GetBuildingList(ByVal viewmodel As ViewModelBuilding, ByVal page As Integer) As ViewModelBuilding
        Dim db As MeetingRoomDBDataContext = New MeetingRoomDBDataContext
        Dim max As Integer = 0
        Dim MaxItem As Integer = viewmodel.PageSize


        ' データを読み込む
        max = (From t In db.GetTable(Of BUILDING)() Order By t.SEQ_ID).Count

        viewmodel.BUILDING = (From t In db.GetTable(Of BUILDING)() Order By t.SEQ_ID).Skip(page * MaxItem).Take(MaxItem)

        viewmodel.ItemCount = max

        Return viewmodel

    End Function

    Public Sub SaveNewBuilding(ByVal viewmodel As ViewModelNewBuilding)

        Using ts = New TransactionScope
            Using db As New MeetingRoomDBDataContext
                Dim record As New BUILDING

                record.BUILDING_NAME = viewmodel.BUILDING_NAME
                record.DISABLED = False

                db.BUILDING.InsertOnSubmit(record)
                db.SubmitChanges()

            End Using
            ts.Complete()
        End Using

    End Sub

    Public Function GetEditBuildingData(ByVal viewmodel As ViewModelEditBuilding) As ViewModelEditBuilding
        Dim db As MeetingRoomDBDataContext = New MeetingRoomDBDataContext

        Dim model As BUILDING = New BUILDING

        model = (From r In db.GetTable(Of BUILDING)() Select r
                                       Where r.SEQ_ID = viewmodel.SEQ_ID).First


        viewmodel.SEQ_ID = model.SEQ_ID
        viewmodel.BUILDING_NAME = model.BUILDING_NAME
        viewmodel.DISABLED = model.DISABLED

        Return viewmodel

    End Function

    Public Sub SaveEditBuilding(ByVal viewmodel As ViewModelEditBuilding)
        Using ts = New TransactionScope
            Using db As New MeetingRoomDBDataContext
                Dim record As New BUILDING

                record = (From r In db.GetTable(Of BUILDING)() Select r
                            Where r.SEQ_ID = viewmodel.SEQ_ID).First

                record.BUILDING_NAME = viewmodel.BUILDING_NAME
                record.DISABLED = viewmodel.DISABLED
                db.SubmitChanges()

            End Using
            ts.Complete()
        End Using

    End Sub

    Public Function SetBuildingList() As IEnumerable(Of SelectListItem)
        Dim db As MeetingRoomDBDataContext = New MeetingRoomDBDataContext

        Dim Building = From t In db.GetTable(Of BUILDING)() Where t.DISABLED = False Order By t.SEQ_ID Ascending

        Dim list As New List(Of SelectListItem)

        For Each item In Building
            list.Add(New SelectListItem() With {.Text = item.BUILDING_NAME, .Value = item.SEQ_ID})
        Next

        Return list.AsEnumerable
    End Function

    Public Function SetRoomList(ByVal id As String) As IEnumerable(Of SelectListItem)
        Dim db As MeetingRoomDBDataContext = New MeetingRoomDBDataContext

        Dim room = From t In db.GetTable(Of ROOM)() Where t.DISABLED = False And t.BUILDING_ID = id Order By t.SEQ_ID Ascending

        Dim list As New List(Of SelectListItem)

        For Each item In room
            list.Add(New SelectListItem() With {.Text = item.ROOM_NAME, .Value = item.SEQ_ID})
        Next

        Return list.AsEnumerable
    End Function

    Public Function SetRoomComment(ByVal id As String) As String
        Dim db As MeetingRoomDBDataContext = New MeetingRoomDBDataContext

        Dim room = From t In db.GetTable(Of ROOM)() Where t.SEQ_ID = id

        Return room.First.COMMENT
    End Function

    Public Function SetRoomStartTime(ByVal id As String) As String
        Dim db As MeetingRoomDBDataContext = New MeetingRoomDBDataContext

        Dim room = From t In db.GetTable(Of ROOM)() Where t.SEQ_ID = id

        Return room.First.START_TIME
    End Function

    Public Function SetRoomEndTime(ByVal id As String) As String
        Dim db As MeetingRoomDBDataContext = New MeetingRoomDBDataContext

        Dim room = From t In db.GetTable(Of ROOM)() Where t.SEQ_ID = id

        Return room.First.END_TIME
    End Function

    Public Function SetBuildingName(ByVal id As Integer) As String
        Dim db As MeetingRoomDBDataContext = New MeetingRoomDBDataContext

        Dim Building = (From t In db.GetTable(Of BUILDING)() Where t.SEQ_ID = id).First

        Return Building.BUILDING_NAME
    End Function

    Public Function SetRoomName(ByVal id As Integer) As String
        Dim db As MeetingRoomDBDataContext = New MeetingRoomDBDataContext

        Dim Room = (From t In db.GetTable(Of ROOM)() Where t.SEQ_ID = id).First

        Return Room.ROOM_NAME
    End Function

    Public Function SetStartEndTime() As IEnumerable(Of SelectListItem)
        Dim StartEndTime As New List(Of SelectListItem)

        ' 予約可能時間を設定(30分おき)
        For i As Integer = 0 To 24
            For j As Integer = 0 To 59 Step 30
                Dim Time As String = String.Format("{0:00}{1:00}", i, j)
                Dim Time2 As String = String.Format("{0:00}:{1:00}", i, j)

                StartEndTime.Add(New SelectListItem() With {.Text = Time2, .Value = Time})

                If i = 24 AndAlso j = 0 Then
                    j = 100
                End If
            Next
        Next

        Return StartEndTime.AsEnumerable
    End Function

    Public Function SetReserveStartTime(ByVal MinTime As String, ByVal MaxTime As String) As IEnumerable(Of SelectListItem)
        Dim StartEndTime As New List(Of SelectListItem)

        ' 予約可能時間を設定(30分おき)
        For i As Integer = 0 To 24
            For j As Integer = 0 To 59 Step 30
                Dim Time As String = String.Format("{0:00}{1:00}", i, j)
                Dim Time2 As String = String.Format("{0:00}:{1:00}", i, j)
                If Time >= MinTime AndAlso Time < MaxTime Then
                    StartEndTime.Add(New SelectListItem() With {.Text = Time2, .Value = Time})
                End If
            Next
        Next

        Return StartEndTime.AsEnumerable
    End Function

    Public Function SetReserveEndTime(ByVal MinTime As String, ByVal MaxTime As String) As IEnumerable(Of SelectListItem)
        Dim StartEndTime As New List(Of SelectListItem)

        ' 予約可能時間を設定(30分おき)
        For i As Integer = 0 To 24
            For j As Integer = 0 To 59 Step 30
                Dim Time As String = String.Format("{0:00}{1:00}", i, j)
                Dim Time2 As String = String.Format("{0:00}:{1:00}", i, j)
                If Time > MinTime AndAlso Time <= MaxTime Then
                    StartEndTime.Add(New SelectListItem() With {.Text = Time2, .Value = Time})
                End If

            Next
        Next

        Return StartEndTime.AsEnumerable
    End Function

    Public Function AlreadyReserve(ByVal viewmodel As ViewModelReservationRoom) As Boolean
        Dim db As New MeetingRoomDBDataContext

        Dim Reservation = From t In db.GetTable(Of RESERVATION)() Where t.BUILDING_ID = viewmodel.BUILDING AndAlso t.ROOM_ID = viewmodel.ROOM AndAlso t.RESERVE_DATE = viewmodel.RESERVE_DATE.Replace("/", "")

        For Each item In Reservation
            Dim reserveStartTime As String = item.RESERVE_START_TIME
            Dim reserveEndTime As String = item.RESERVE_END_TIME

            Dim hash As Hashtable = New Hashtable
            ' 現在予約されているデータの予約開始時間から予約終了時間までフラグを立てる
            For i As Integer = 0 To 24
                For j As Integer = 0 To 59 Step 30
                    Dim tmp As String = String.Format("{0:00}{1:00}", i, j)

                    If tmp >= reserveStartTime AndAlso tmp < reserveEndTime Then
                        hash.Item(tmp) = 1
                    Else
                        hash.Item(tmp) = 0
                    End If
                Next
            Next

            Dim hash2 As Hashtable = New Hashtable

            ' 予約しようとしているデータの予約開始時間から予約終了時間までフラグを立てる
            For i As Integer = 0 To 24
                For j As Integer = 0 To 59 Step 30
                    Dim tmp As String = String.Format("{0:00}{1:00}", i, j)

                    If tmp >= viewmodel.START_TIME AndAlso tmp < viewmodel.END_TIME Then
                        hash2.Item(tmp) = 1
                    Else
                        hash2.Item(tmp) = 0
                    End If
                Next
            Next

            ' 予約されているがどうか判定する
            For Each key In hash.Keys
                If hash.Item(key) = 1 AndAlso hash2.Item(key) = 1 Then
                    Return True
                End If
            Next
        Next

        Return False

    End Function

    Public Function GetRoomList(ByVal viewmodel As ViewModelRoom, ByVal page As Integer) As ViewModelRoom
        Dim db As MeetingRoomDBDataContext = New MeetingRoomDBDataContext
        Dim max As Integer = 0
        Dim MaxItem As Integer = viewmodel.PageSize

        ' データを読み込む
        max = (From t In db.GetTable(Of ROOM)() Where t.BUILDING_ID = viewmodel.BUILDING Order By t.SEQ_ID).Count

        viewmodel.ROOM = (From t In db.GetTable(Of ROOM)() Where t.BUILDING_ID = viewmodel.BUILDING Order By t.SEQ_ID).Skip(page * MaxItem).Take(MaxItem)

        viewmodel.ItemCount = max

        Return viewmodel

    End Function

    Public Sub SaveNewRoom(ByVal viewmodel As ViewModelNewRoom)

        Using ts = New TransactionScope
            Using db As New MeetingRoomDBDataContext
                Dim record As New ROOM

                record.ROOM_NAME = viewmodel.ROOM_NAME
                record.COMMENT = viewmodel.COMMENT
                record.BUILDING_ID = viewmodel.BUILDING
                record.START_TIME = viewmodel.START_TIME
                record.END_TIME = viewmodel.END_TIME
                record.DISABLED = False

                db.ROOM.InsertOnSubmit(record)
                db.SubmitChanges()

            End Using
            ts.Complete()
        End Using

    End Sub

    Public Function GetEditRoomData(ByVal viewmodel As ViewModelEditRoom) As ViewModelEditRoom
        Dim db As MeetingRoomDBDataContext = New MeetingRoomDBDataContext

        Dim model As ROOM = New ROOM

        model = (From r In db.GetTable(Of ROOM)() Select r
                                       Where r.SEQ_ID = viewmodel.SEQ_ID).First


        viewmodel.SEQ_ID = model.SEQ_ID
        viewmodel.ROOM_NAME = model.ROOM_NAME
        viewmodel.COMMENT = model.COMMENT
        viewmodel.START_TIME = model.START_TIME
        viewmodel.END_TIME = model.END_TIME
        viewmodel.DISABLED = model.DISABLED

        Return viewmodel

    End Function

    Public Sub SaveEditRoom(ByVal viewmodel As ViewModelEditRoom)
        Using ts = New TransactionScope
            Using db As New MeetingRoomDBDataContext
                Dim record As New ROOM

                record = (From r In db.GetTable(Of ROOM)() Select r
                            Where r.SEQ_ID = viewmodel.SEQ_ID).First

                record.ROOM_NAME = viewmodel.ROOM_NAME
                record.COMMENT = viewmodel.COMMENT
                record.BUILDING_ID = viewmodel.BUILDING
                record.START_TIME = viewmodel.START_TIME
                record.END_TIME = viewmodel.END_TIME
                record.DISABLED = viewmodel.DISABLED
                db.SubmitChanges()

            End Using
            ts.Complete()
        End Using

    End Sub

    Public Sub SaveReservation(ByVal viewmodel As ViewModelReservationRoom)
        Using ts = New TransactionScope
            Using db As New MeetingRoomDBDataContext
                Dim record As New RESERVATION

                record.BUILDING_ID = viewmodel.BUILDING
                record.ROOM_ID = viewmodel.ROOM
                record.BUILDING_ID = viewmodel.BUILDING
                record.RESERVE_PERSON = viewmodel.RESERVE_PERSON
                record.RESERVE_DATE = viewmodel.RESERVE_DATE.Replace("/", "")
                record.RESERVE_START_TIME = viewmodel.START_TIME
                record.RESERVE_END_TIME = viewmodel.END_TIME

                db.RESERVATION.InsertOnSubmit(record)
                db.SubmitChanges()

            End Using
            ts.Complete()
        End Using

    End Sub

    Public Function GetReservation(ByVal viewmodel As ViewModelReservationConfirm) As IEnumerable(Of ReservedInfo)
        Dim db As New MeetingRoomDBDataContext

        Dim room = From t In db.GetTable(Of ROOM)() Where t.BUILDING_ID = viewmodel.BUILDING AndAlso t.DISABLED = False

        Dim list = From t In db.GetTable(Of RESERVATION)()
                   Join t2 In db.GetTable(Of BUILDING)() On t.BUILDING_ID Equals t2.SEQ_ID
                   Join t3 In db.GetTable(Of ROOM)() On t.ROOM_ID Equals t3.SEQ_ID
                   Where t.BUILDING_ID = viewmodel.BUILDING AndAlso t.RESERVE_DATE = viewmodel.RESERVE_DATE.Replace("/", "") AndAlso t2.DISABLED = False AndAlso t3.DISABLED = False

        Dim ary = New List(Of ReservedInfo)

        ' 会議室ごとに処理を行う
        ' 予約時間帯の計算
        ' 予約されているかフラグのセット
        For Each item In room
            Dim roomID As Integer = item.SEQ_ID
            Dim list2 = From t In list Where t.t.ROOM_ID = roomID

            Dim obj As ReservedInfo = New ReservedInfo

            obj.RoomID = roomID
            obj.RoomName = item.ROOM_NAME
            obj.ReserveDate = viewmodel.RESERVE_DATE.Replace("/", "")

            ' 予約済みの時間帯を計算
            For Each item2 In list2
                Dim reserveStartTime As String = item2.t.RESERVE_START_TIME
                Dim reserveEndTime As String = item2.t.RESERVE_END_TIME

                For i As Integer = 0 To 24
                    For j As Integer = 0 To 59 Step 30
                        Dim tmp As String = String.Format("{0:00}{1:00}", i, j)

                        If tmp >= reserveStartTime AndAlso tmp < reserveEndTime Then
                            obj.ReservedState.Item(tmp) = 1
                        Else
                            If obj.ReservedState.Item(tmp) <> 1 Then
                                obj.ReservedState.Item(tmp) = 0
                            End If
                        End If
                    Next
                Next
            Next

            ' 予約済みが一箇所でもあればフラグをセットする
            obj.IsReserved = False
            For Each key As String In obj.ReservedState.Keys
                If obj.ReservedState.Item(key) = 1 Then
                    obj.IsReserved = True
                End If
            Next
            ary.Add(obj)
        Next

        Return ary.AsEnumerable
    End Function

    Public Function GetReservationList(ByVal id As Integer, ByVal d As String) As IEnumerable(Of ReservationData)
        Dim db As New MeetingRoomDBDataContext

        ' 予約の一覧を取得
        Dim list = From t In db.GetTable(Of RESERVATION)()
                   Join t2 In db.GetTable(Of BUILDING)() On t.BUILDING_ID Equals t2.SEQ_ID
                   Join t3 In db.GetTable(Of ROOM)() On t.ROOM_ID Equals t3.SEQ_ID
                   Where t.ROOM_ID = id AndAlso t.RESERVE_DATE = d Order By t.RESERVE_START_TIME Ascending

        Dim array As New List(Of ReservationData)

        For Each item In list
            Dim obj As New ReservationData

            obj.SEQ_ID = item.t.SEQ_ID
            obj.Building = item.t2.BUILDING_NAME
            obj.Room = item.t3.ROOM_NAME
            obj.Person = item.t.RESERVE_PERSON
            obj.ReserveDate = item.t.RESERVE_DATE
            obj.ReserveStartTime = item.t.RESERVE_START_TIME
            obj.ReserveEndTime = item.t.RESERVE_END_TIME

            array.Add(obj)
        Next

        Return array.AsEnumerable
    End Function

    Public Sub CancelReservation(ByVal id As String)
        Using ts = New TransactionScope
            Using db As New MeetingRoomDBDataContext
                Dim record As New RESERVATION

                record = (From r In db.GetTable(Of RESERVATION)() Select r
                            Where r.SEQ_ID = id).First

                db.RESERVATION.DeleteOnSubmit(record)
                db.SubmitChanges()
            End Using
            ts.Complete()
        End Using

    End Sub
End Class
