﻿Imports System.Transactions

Public Class UserService
    Public Function GetList(ByVal viewmodel As ViewModelUser, ByVal page As Integer) As ViewModelUser
        Dim db As MeetingRoomDBDataContext = New MeetingRoomDBDataContext
        Dim max As Integer = 0
        Dim MaxItem As Integer = viewmodel.PageSize


        ' データを読み込む
        max = (From u In db.GetTable(Of MST_USER)()
                   Where viewmodel.OPERATOR_ID Is Nothing OrElse u.OPERATOR_ID.Contains(viewmodel.OPERATOR_ID)
                   Order By u.OPERATOR_ID
                   Select New UserData With {
                       .OPERATOR_ID = u.OPERATOR_ID,
                       .OPERATOR_NAME = u.OPERATOR_NAME,
                       .ROLE = db.GetUserRole(u.OPERATOR_ID),
                       .DATE_CREATED = u.DATE_CREATED,
                        .DISABLED = If(u.DISABLED = True, "True", "False"),
                        .LOCK = If(u.LOCK = True, "True", "False")
                   }).Count

        viewmodel.MST_USER = (From u In db.GetTable(Of MST_USER)()
                   Where viewmodel.OPERATOR_ID Is Nothing OrElse u.OPERATOR_ID.Contains(viewmodel.OPERATOR_ID)
                   Order By u.OPERATOR_ID
                   Select New UserData With {
                       .OPERATOR_ID = u.OPERATOR_ID,
                       .OPERATOR_NAME = u.OPERATOR_NAME,
                       .ROLE = db.GetUserRole(u.OPERATOR_ID),
                       .DATE_CREATED = u.DATE_CREATED,
                        .DISABLED = If(u.DISABLED = True, "True", "False"),
                       .LOCK = If(u.LOCK = True, "True", "False")
                   }).Skip(page * MaxItem).Take(MaxItem)


        viewmodel.ItemCount = max

        Return viewmodel
    End Function

    Public Function GetNewUserData(ByVal viewmodel As ViewModelNewUser) As ViewModelNewUser
        Dim db As MeetingRoomDBDataContext = New MeetingRoomDBDataContext

        Dim MstRole = (From u In db.GetTable(Of MST_ROLE)()
                       Order By u.ROLE_ID)
        viewmodel.ROLE_CHECKBOXLIST = New List(Of SelectListItem)
        For Each item In MstRole.AsEnumerable
            viewmodel.ROLE_CHECKBOXLIST.Add(New SelectListItem() With {.Text = item.ROLE_NAME, .Value = item.ROLE_ID})
        Next

        Return viewmodel

    End Function

    Public Sub SaveNewUser(ByVal viewmodel As ViewModelNewUser)
        Dim encryptPassword As String = Crypt.EncryptSHA256(viewmodel.PASSWORD)

        Using ts = New TransactionScope
            Using db As New MeetingRoomDBDataContext
                Dim record As New MST_USER

                record.OPERATOR_ID = viewmodel.OPERATOR_ID
                record.OPERATOR_NAME = viewmodel.OPERATOR_NAME
                record.ENCRYPTED_PASSWORD = encryptPassword
                record.INITIAL_PASSWORD = True
                record.LOCK = False
                record.FAILED_COUNT = 0
                record.DATE_CREATED = Now.ToString("yyyy/MM/dd HH:mm:ss")
                record.DISABLED = False

                db.MST_USER.InsertOnSubmit(record)
                db.SubmitChanges()

                For Each item In viewmodel.ROLE_CHECKBOXLIST.AsEnumerable
                    If viewmodel.ROLE(item.Value - 1) Then
                        Dim record2 As New MAP_ROLE

                        record2.ROLE_ID = item.Value
                        record2.OPERATOR_ID = viewmodel.OPERATOR_ID

                        db.MAP_ROLE.InsertOnSubmit(record2)
                        db.SubmitChanges()
                    End If
                Next


            End Using
            ts.Complete()
        End Using

    End Sub

    Public Function GetEditUserData(ByVal viewmodel As ViewModelEditUser) As ViewModelEditUser
        Dim db As MeetingRoomDBDataContext = New MeetingRoomDBDataContext

        Dim MstRole = (From u In db.GetTable(Of MST_ROLE)()
                       Order By u.ROLE_ID)
        viewmodel.ROLE_CHECKBOXLIST = New List(Of SelectListItem)
        For Each item In MstRole.AsEnumerable
            viewmodel.ROLE_CHECKBOXLIST.Add(New SelectListItem() With {.Text = item.ROLE_NAME, .Value = item.ROLE_ID})
        Next

        Dim model As MST_USER = New MST_USER

        model = (From r In db.GetTable(Of MST_USER)() Select r
                                       Where r.OPERATOR_ID = viewmodel.OPERATOR_ID).First


        viewmodel.OPERATOR_ID = model.OPERATOR_ID
        viewmodel.OPERATOR_NAME = model.OPERATOR_NAME
        viewmodel.LOCK = model.LOCK
        viewmodel.DISABLED = model.DISABLED

        Dim model2 = (From r In db.GetTable(Of MAP_ROLE)() Select r
                                       Where r.OPERATOR_ID = viewmodel.OPERATOR_ID)

        Dim count As Integer = viewmodel.ROLE_CHECKBOXLIST.Count
        ReDim viewmodel.ROLE(count - 1)

        For i As Integer = 0 To viewmodel.ROLE.Count - 1
            viewmodel.ROLE(i) = False
        Next

        For Each item In model2
            Select Case item.ROLE_ID
                Case 1
                    viewmodel.ROLE(0) = True
                Case 2
                    viewmodel.ROLE(1) = True
                Case 3
                    viewmodel.ROLE(2) = True
            End Select

        Next

        Return viewmodel

    End Function

    Public Function GetEditUserDataForConfirm(ByVal viewmodel As ViewModelEditUser) As ViewModelEditUser
        Dim db As MeetingRoomDBDataContext = New MeetingRoomDBDataContext

        Dim MstRole = (From u In db.GetTable(Of MST_ROLE)()
                       Order By u.ROLE_ID)
        viewmodel.ROLE_CHECKBOXLIST = New List(Of SelectListItem)
        For Each item In MstRole.AsEnumerable
            viewmodel.ROLE_CHECKBOXLIST.Add(New SelectListItem() With {.Text = item.ROLE_NAME, .Value = item.ROLE_ID})
        Next

        Return viewmodel

    End Function

    Public Sub SaveEditUser(ByVal viewmodel As ViewModelEditUser)
        Using ts = New TransactionScope
            Using db As New MeetingRoomDBDataContext
                Dim record As New MST_USER

                record = (From r In db.GetTable(Of MST_USER)() Select r
                            Where r.OPERATOR_ID = viewmodel.OPERATOR_ID).First

                record.OPERATOR_NAME = viewmodel.OPERATOR_NAME
                record.LOCK = viewmodel.LOCK
                record.DISABLED = viewmodel.DISABLED
                db.SubmitChanges()

                Dim record2 = (From r In db.GetTable(Of MAP_ROLE)() Select r
                            Where r.OPERATOR_ID = viewmodel.OPERATOR_ID)

                db.MAP_ROLE.DeleteAllOnSubmit(record2)


                For Each item In viewmodel.ROLE_CHECKBOXLIST.AsEnumerable
                    If viewmodel.ROLE(item.Value - 1) Then
                        Dim record3 As New MAP_ROLE

                        record3.ROLE_ID = item.Value
                        record3.OPERATOR_ID = viewmodel.OPERATOR_ID

                        db.MAP_ROLE.InsertOnSubmit(record3)
                        db.SubmitChanges()
                    End If
                Next


            End Using
            ts.Complete()
        End Using

    End Sub

    Public Sub SaveEditPassword(ByVal viewmodel As ViewModelEditPassword)
        Dim encryptPassword As String = Crypt.EncryptSHA256(viewmodel.PASSWORD)

        Using ts = New TransactionScope
            Using db As New MeetingRoomDBDataContext
                Dim record As New MST_USER

                record = (From r In db.GetTable(Of MST_USER)() Select r
                            Where r.OPERATOR_ID = viewmodel.OPERATOR_ID).First

                record.ENCRYPTED_PASSWORD = encryptPassword
                record.LAST_PASSWORD_CHANGE = Now.ToString("yyyy/MM/dd HH:mm:ss")
                record.INITIAL_PASSWORD = True
                record.LOCK = False
                record.FAILED_COUNT = 0

                db.SubmitChanges()

            End Using
            ts.Complete()
        End Using

    End Sub

    Public Function ExistsOperatorID(ByVal operatorId As String) As Boolean
        Dim db As MeetingRoomDBDataContext = New MeetingRoomDBDataContext()

        Dim user = From t In db.GetTable(Of MST_USER)() Where t.OPERATOR_ID = operatorId

        If user.Count > 0 Then
            Return True
        Else
            Return False
        End If
    End Function

    Public Function CorrectPassword(ByVal operatorId As String, ByVal password As String) As Boolean
        Dim encryptPassword As String = Crypt.EncryptSHA256(password)

        Dim db As MeetingRoomDBDataContext = New MeetingRoomDBDataContext()

        Dim user = From t In db.GetTable(Of MST_USER)() Where t.OPERATOR_ID = operatorId AndAlso t.ENCRYPTED_PASSWORD = encryptPassword

        If user.Count > 0 Then
            Return True
        Else
            Return False
        End If

    End Function
End Class
