﻿Imports System.ComponentModel.DataAnnotations
Imports System.Reflection

<AttributeUsage(AttributeTargets.Property, AllowMultiple:=False)>
Public Class IsMailAttribute
    Inherits ValidationAttribute
    Implements IClientValidatable

    Public Overrides Function IsValid(ByVal value As Object) As Boolean

        If value Is Nothing Then Return True

        Dim reg As Regex = New Regex("^[a-zA-Z_0-9]+([-+.][a-zA-Z_0-9]+)*@[a-zA-Z_0-9]+([-.][a-zA-Z_0-9]+)*\.[a-zA-Z_0-9]+([-.][a-zA-Z_0-9]+)*$")

        Return reg.IsMatch(value)
    End Function

    Public Function GetClientValidationRules(ByVal metadata As System.Web.Mvc.ModelMetadata, ByVal context As System.Web.Mvc.ControllerContext) As System.Collections.Generic.IEnumerable(Of System.Web.Mvc.ModelClientValidationRule) Implements System.Web.Mvc.IClientValidatable.GetClientValidationRules

        Dim rule = New ModelClientValidationRule()

        rule.ValidationType = "ismail"
        rule.ErrorMessage = FormatErrorMessage(metadata.GetDisplayName())
        rule.ValidationParameters("opts") = ""

        Dim list = New List(Of ModelClientValidationRule)()
        list.Add(rule)
        Return list
    End Function


End Class

<AttributeUsage(AttributeTargets.Property, AllowMultiple:=False)>
Public Class IncludeHalfCharacterAttribute
    Inherits ValidationAttribute
    Implements IClientValidatable
    Public Overrides Function IsValid(ByVal value As Object) As Boolean

        If value Is Nothing Then Return True

        Dim reg As Regex = New Regex("[ -~｡-ﾟ]+")


        Return Not reg.IsMatch(value)
    End Function

    Public Function GetClientValidationRules(ByVal metadata As System.Web.Mvc.ModelMetadata, ByVal context As System.Web.Mvc.ControllerContext) As System.Collections.Generic.IEnumerable(Of System.Web.Mvc.ModelClientValidationRule) Implements System.Web.Mvc.IClientValidatable.GetClientValidationRules

        Dim rule = New ModelClientValidationRule()

        rule.ValidationType = "includehalfcharacter"
        rule.ErrorMessage = FormatErrorMessage(metadata.GetDisplayName())
        rule.ValidationParameters("opts") = ""

        Dim list = New List(Of ModelClientValidationRule)()
        list.Add(rule)
        Return list
    End Function
End Class

<AttributeUsage(AttributeTargets.Property, AllowMultiple:=False)>
Public Class IsNumericAttribute
    Inherits ValidationAttribute
    Implements IClientValidatable

    Public Overrides Function IsValid(ByVal value As Object) As Boolean

        If value Is Nothing Then Return True

        Dim reg As Regex = New Regex("^[0-9]*$")


        Return reg.IsMatch(value)
    End Function

    Public Function GetClientValidationRules(ByVal metadata As System.Web.Mvc.ModelMetadata, ByVal context As System.Web.Mvc.ControllerContext) As System.Collections.Generic.IEnumerable(Of System.Web.Mvc.ModelClientValidationRule) Implements System.Web.Mvc.IClientValidatable.GetClientValidationRules

        Dim rule = New ModelClientValidationRule()

        rule.ValidationType = "isnumeric"
        rule.ErrorMessage = FormatErrorMessage(metadata.GetDisplayName())
        rule.ValidationParameters("opts") = ""

        Dim list = New List(Of ModelClientValidationRule)()
        list.Add(rule)
        Return list
    End Function

End Class

<AttributeUsage(AttributeTargets.Property, AllowMultiple:=False)>
Public Class IsAlphaNumericAttribute
    Inherits ValidationAttribute
    Implements IClientValidatable

    Public Overrides Function IsValid(ByVal value As Object) As Boolean

        If value Is Nothing Then Return True

        Dim reg As Regex = New Regex("^[0-9a-zA-Z]*$")


        Return reg.IsMatch(value)
    End Function

    Public Function GetClientValidationRules(ByVal metadata As System.Web.Mvc.ModelMetadata, ByVal context As System.Web.Mvc.ControllerContext) As System.Collections.Generic.IEnumerable(Of System.Web.Mvc.ModelClientValidationRule) Implements System.Web.Mvc.IClientValidatable.GetClientValidationRules

        Dim rule = New ModelClientValidationRule()

        rule.ValidationType = "isalphanumeric"
        rule.ErrorMessage = FormatErrorMessage(metadata.GetDisplayName())
        rule.ValidationParameters("opts") = ""

        Dim list = New List(Of ModelClientValidationRule)()
        list.Add(rule)
        Return list
    End Function

End Class

<AttributeUsage(AttributeTargets.Property, AllowMultiple:=False)>
Public Class IsKatakanaAttribute
    Inherits ValidationAttribute
    Implements IClientValidatable

    Public Overrides Function IsValid(ByVal value As Object) As Boolean

        If value Is Nothing Then Return True

        Dim reg As Regex = New Regex("^[ァ-ヶ|ー]+$")


        Return reg.IsMatch(value)
    End Function


    Public Function GetClientValidationRules(metadata As ModelMetadata, context As ControllerContext) As IEnumerable(Of ModelClientValidationRule) Implements IClientValidatable.GetClientValidationRules
        Dim rule = New ModelClientValidationRule()

        rule.ValidationType = "iskatakana"
        rule.ErrorMessage = FormatErrorMessage(metadata.GetDisplayName())
        rule.ValidationParameters("opts") = ""

        Dim list = New List(Of ModelClientValidationRule)()
        list.Add(rule)
        Return list
    End Function
End Class

<AttributeUsage(AttributeTargets.Property, AllowMultiple:=False)>
Public Class IncludeSpecialCharacterAttribute
    Inherits ValidationAttribute
    Implements IClientValidatable

    Public Overrides Function IsValid(ByVal value As Object) As Boolean

        If value Is Nothing Then Return True

        Dim reg As Regex = New Regex("[①②③④⑤⑥⑦⑧⑨⑩⑪⑫⑬⑭⑮⑯⑰⑱⑲⑳ⅠⅡⅢⅣⅤ≡∑∫∮√⊥∠∟⊿∵∩∪№㏍℡㊤㊥㊦㊧㊨㈱㈲㈹㍾㍽㍼㍻㍉㎜㎝㎞㎎㎏㏄㍉㌔㌢㍍㌘㌧㌃㌶㍑㍗㌍㌦㌣㌫㍊㌻ⅠⅡⅢⅣⅤⅥⅦⅧⅨⅩ]+")


        Return Not reg.IsMatch(value)
    End Function

    Public Function GetClientValidationRules(metadata As ModelMetadata, context As ControllerContext) As IEnumerable(Of ModelClientValidationRule) Implements IClientValidatable.GetClientValidationRules
        Dim rule = New ModelClientValidationRule()

        rule.ValidationType = "includespecialcharacter"
        rule.ErrorMessage = FormatErrorMessage(metadata.GetDisplayName())
        rule.ValidationParameters("opts") = ""

        Dim list = New List(Of ModelClientValidationRule)()
        list.Add(rule)
        Return list
    End Function
End Class

<AttributeUsage(AttributeTargets.Property, AllowMultiple:=False)>
Public Class IncludeZenkakuAlphabetAttribute
    Inherits ValidationAttribute
    Implements IClientValidatable

    Public Overrides Function IsValid(ByVal value As Object) As Boolean

        If value Is Nothing Then Return True

        Dim reg As Regex = New Regex("[ａ-ｚＡ-Ｚ]+")


        Return Not reg.IsMatch(value)
    End Function


    Public Function GetClientValidationRules(metadata As ModelMetadata, context As ControllerContext) As IEnumerable(Of ModelClientValidationRule) Implements IClientValidatable.GetClientValidationRules
        Dim rule = New ModelClientValidationRule()

        rule.ValidationType = "includezenkakualphabet"
        rule.ErrorMessage = FormatErrorMessage(metadata.GetDisplayName())
        rule.ValidationParameters("opts") = ""

        Dim list = New List(Of ModelClientValidationRule)()
        list.Add(rule)
        Return list
    End Function
End Class

<AttributeUsage(AttributeTargets.Property, AllowMultiple:=False)>
Public Class IncludeZenkakuNumericAttribute
    Inherits ValidationAttribute
    Implements IClientValidatable

    Public Overrides Function IsValid(ByVal value As Object) As Boolean

        If value Is Nothing Then Return True

        Dim reg As Regex = New Regex("[０-９]+")


        Return Not reg.IsMatch(value)
    End Function


    Public Function GetClientValidationRules(metadata As ModelMetadata, context As ControllerContext) As IEnumerable(Of ModelClientValidationRule) Implements IClientValidatable.GetClientValidationRules
        Dim rule = New ModelClientValidationRule()

        rule.ValidationType = "includezenkakunumeric"
        rule.ErrorMessage = FormatErrorMessage(metadata.GetDisplayName())
        rule.ValidationParameters("opts") = ""

        Dim list = New List(Of ModelClientValidationRule)()
        list.Add(rule)
        Return list
    End Function
End Class


<AttributeUsage(AttributeTargets.Property, AllowMultiple:=True)>
Public Class SameValueAttribute
    Inherits ValidationAttribute
    Implements IClientValidatable


    Public Property Value1 As String
    Public Property Value2 As String

    Public Sub New(ByVal argValue1 As String, ByVal argValue2 As String)
        Me.Value1 = argValue1
        Me.Value2 = argValue2
    End Sub

    Protected Overrides Function IsValid(ByVal value As Object, ByVal validationContext As ValidationContext) As ValidationResult
        Dim piMail1 As PropertyInfo = validationContext.ObjectType.GetProperty(Value1)
        Dim piMail2 As PropertyInfo = validationContext.ObjectType.GetProperty(Value2)

        If piMail1.GetValue(validationContext.ObjectInstance, Nothing) = piMail2.GetValue(validationContext.ObjectInstance, Nothing) Then
            Return Nothing
        Else
            Return New ValidationResult("")
        End If
    End Function

    Public Function GetClientValidationRules(metadata As ModelMetadata, context As ControllerContext) As IEnumerable(Of ModelClientValidationRule) Implements IClientValidatable.GetClientValidationRules
        Dim rule = New ModelClientValidationRule()

        rule.ValidationType = "samevalue"
        rule.ErrorMessage = FormatErrorMessage(metadata.GetDisplayName())
        rule.ValidationParameters("opts1") = Me.Value1
        rule.ValidationParameters("opts2") = Me.Value2

        Dim list = New List(Of ModelClientValidationRule)()
        list.Add(rule)
        Return list
    End Function
End Class

<AttributeUsage(AttributeTargets.Property, AllowMultiple:=False)>
Public Class IsDateAttribute
    Inherits ValidationAttribute
    Implements IClientValidatable

    Public Overrides Function IsValid(ByVal value As Object) As Boolean

        If value Is Nothing Then Return True

        Return IsDate(value)
    End Function

    Public Function GetClientValidationRules(metadata As ModelMetadata, context As ControllerContext) As IEnumerable(Of ModelClientValidationRule) Implements IClientValidatable.GetClientValidationRules
        Dim rule = New ModelClientValidationRule()

        rule.ValidationType = "isdate"
        rule.ErrorMessage = FormatErrorMessage(metadata.GetDisplayName())
        rule.ValidationParameters("opts") = ""

        Dim list = New List(Of ModelClientValidationRule)()
        list.Add(rule)
        Return list
    End Function
End Class

<AttributeUsage(AttributeTargets.Property, AllowMultiple:=True)>
Public Class TelAndMTelValidatorAttribute
    Inherits ValidationAttribute
    Implements IClientValidatable

    Public Property TEL1 As String
    Public Property TEL2 As String
    Public Property TEL3 As String
    Public Property M_TEL1 As String
    Public Property M_TEL2 As String
    Public Property M_TEL3 As String

    Public Sub New(ByVal argTEL1 As String, ByVal argTEL2 As String, ByVal argTEL3 As String, _
                    ByVal argM_TEL1 As String, ByVal argM_TEL2 As String, ByVal argM_TEL3 As String)
        Me.TEL1 = argTEL1
        Me.TEL2 = argTEL2
        Me.TEL3 = argTEL3

        Me.M_TEL1 = argM_TEL1
        Me.M_TEL2 = argM_TEL2
        Me.M_TEL3 = argM_TEL3

    End Sub

    Protected Overrides Function IsValid(ByVal value As Object, ByVal validationContext As ValidationContext) As ValidationResult
        Dim piTEL1 As PropertyInfo = validationContext.ObjectType.GetProperty(TEL1)
        Dim piTEL2 As PropertyInfo = validationContext.ObjectType.GetProperty(TEL2)
        Dim piTEL3 As PropertyInfo = validationContext.ObjectType.GetProperty(TEL3)
        Dim piMTEL1 As PropertyInfo = validationContext.ObjectType.GetProperty(M_TEL1)
        Dim piMTEL2 As PropertyInfo = validationContext.ObjectType.GetProperty(M_TEL2)
        Dim piMTEL3 As PropertyInfo = validationContext.ObjectType.GetProperty(M_TEL3)

        Dim propList As New List(Of String)

        If piTEL1.GetValue(validationContext.ObjectInstance, Nothing) = "" AndAlso _
            piTEL2.GetValue(validationContext.ObjectInstance, Nothing) = "" AndAlso _
            piTEL3.GetValue(validationContext.ObjectInstance, Nothing) = "" AndAlso _
            piMTEL1.GetValue(validationContext.ObjectInstance, Nothing) = "" AndAlso _
            piMTEL2.GetValue(validationContext.ObjectInstance, Nothing) = "" AndAlso _
            piMTEL3.GetValue(validationContext.ObjectInstance, Nothing) = "" Then

            propList.Add(Me.TEL1)
            propList.Add(Me.TEL2)
            propList.Add(Me.TEL3)
            propList.Add(Me.M_TEL1)
            propList.Add(Me.M_TEL2)
            propList.Add(Me.M_TEL3)

            Return New ValidationResult("自宅電話番号か携帯電話番号を入力してください。", propList.AsEnumerable())
        Else
            Return Nothing
        End If
    End Function

    Public Function GetClientValidationRules(metadata As ModelMetadata, context As ControllerContext) As IEnumerable(Of ModelClientValidationRule) Implements IClientValidatable.GetClientValidationRules
        Dim rule = New ModelClientValidationRule()

        rule.ValidationType = "telandmtelvalidator"
        rule.ErrorMessage = "自宅電話番号か携帯電話番号を入力してください。"
        rule.ValidationParameters("opts1") = Me.TEL1
        rule.ValidationParameters("opts2") = Me.TEL2
        rule.ValidationParameters("opts3") = Me.TEL3
        rule.ValidationParameters("opts4") = Me.M_TEL1
        rule.ValidationParameters("opts5") = Me.M_TEL2
        rule.ValidationParameters("opts6") = Me.M_TEL3

        Dim list = New List(Of ModelClientValidationRule)()
        list.Add(rule)
        Return list
    End Function
End Class

<AttributeUsage(AttributeTargets.Property, AllowMultiple:=True)>
Public Class TelValidatorAttribute
    Inherits ValidationAttribute
    Implements IClientValidatable

    Public Property TEL1 As String
    Public Property TEL2 As String
    Public Property TEL3 As String

    Public Sub New(ByVal argTEL1 As String, ByVal argTEL2 As String, ByVal argTEL3 As String)
        Me.TEL1 = argTEL1
        Me.TEL2 = argTEL2
        Me.TEL3 = argTEL3
    End Sub

    Protected Overrides Function IsValid(ByVal value As Object, ByVal validationContext As ValidationContext) As ValidationResult
        Dim piTEL1 As PropertyInfo = validationContext.ObjectType.GetProperty(TEL1)
        Dim piTEL2 As PropertyInfo = validationContext.ObjectType.GetProperty(TEL2)
        Dim piTEL3 As PropertyInfo = validationContext.ObjectType.GetProperty(TEL3)
        Dim propList As New List(Of String)

        If piTEL1.GetValue(validationContext.ObjectInstance, Nothing) <> "" OrElse _
                piTEL2.GetValue(validationContext.ObjectInstance, Nothing) <> "" OrElse _
                piTEL3.GetValue(validationContext.ObjectInstance, Nothing) <> "" Then
            If piTEL1.GetValue(validationContext.ObjectInstance, Nothing) = "" AndAlso validationContext.DisplayName = piTEL1.Name Then
                propList.Add(Me.TEL1)
                Return New ValidationResult("自宅電話番号（市外局番）を入力してください。", propList.AsEnumerable())
            End If
            If piTEL2.GetValue(validationContext.ObjectInstance, Nothing) = "" AndAlso validationContext.DisplayName = piTEL2.Name Then
                propList.Add(Me.TEL2)
                Return New ValidationResult("自宅電話番号（局番）を入力してください。", propList.AsEnumerable())
            End If
            If piTEL3.GetValue(validationContext.ObjectInstance, Nothing) = "" AndAlso validationContext.DisplayName = piTEL3.Name Then
                propList.Add(Me.TEL3)
                Return New ValidationResult("自宅電話番号（枝番）を入力してください。", propList.AsEnumerable())
            End If
        End If

        Return Nothing

    End Function

    Public Function GetClientValidationRules(metadata As ModelMetadata, context As ControllerContext) As IEnumerable(Of ModelClientValidationRule) Implements IClientValidatable.GetClientValidationRules
        Dim rule = New ModelClientValidationRule()

        rule.ValidationType = "telvalidator"
        rule.ErrorMessage = ""
        rule.ValidationParameters("opts1") = Me.TEL1
        rule.ValidationParameters("opts2") = Me.TEL2
        rule.ValidationParameters("opts3") = Me.TEL3

        Dim list = New List(Of ModelClientValidationRule)()
        list.Add(rule)
        Return list
    End Function
End Class

<AttributeUsage(AttributeTargets.Property, AllowMultiple:=True)>
Public Class MTelValidatorAttribute
    Inherits ValidationAttribute
    Implements IClientValidatable

    Public Property M_TEL1 As String
    Public Property M_TEL2 As String
    Public Property M_TEL3 As String

    Public Sub New(ByVal argM_TEL1 As String, ByVal argM_TEL2 As String, ByVal argM_TEL3 As String)
        Me.M_TEL1 = argM_TEL1
        Me.M_TEL2 = argM_TEL2
        Me.M_TEL3 = argM_TEL3


    End Sub

    Protected Overrides Function IsValid(ByVal value As Object, ByVal validationContext As ValidationContext) As ValidationResult
        Dim piM_TEL1 As PropertyInfo = validationContext.ObjectType.GetProperty(M_TEL1)
        Dim piM_TEL2 As PropertyInfo = validationContext.ObjectType.GetProperty(M_TEL2)
        Dim piM_TEL3 As PropertyInfo = validationContext.ObjectType.GetProperty(M_TEL3)
        Dim propList As New List(Of String)

        If piM_TEL1.GetValue(validationContext.ObjectInstance, Nothing) <> "" OrElse _
                piM_TEL2.GetValue(validationContext.ObjectInstance, Nothing) <> "" OrElse _
                piM_TEL3.GetValue(validationContext.ObjectInstance, Nothing) <> "" Then
            If piM_TEL1.GetValue(validationContext.ObjectInstance, Nothing) = "" AndAlso validationContext.DisplayName = piM_TEL1.Name Then
                propList.Add(Me.M_TEL1)
                Return New ValidationResult("携帯電話番号（先頭３桁）を入力してください。", propList.AsEnumerable())
            End If
            If piM_TEL2.GetValue(validationContext.ObjectInstance, Nothing) = "" AndAlso validationContext.DisplayName = piM_TEL2.Name Then
                propList.Add(Me.M_TEL2)
                Return New ValidationResult("携帯電話番号（中央４桁）を入力してください。", propList.AsEnumerable())
            End If
            If piM_TEL3.GetValue(validationContext.ObjectInstance, Nothing) = "" AndAlso validationContext.DisplayName = piM_TEL3.Name Then
                propList.Add(Me.M_TEL3)
                Return New ValidationResult("携帯電話番号（末尾４桁）を入力してください。", propList.AsEnumerable())
            End If
        End If

        Return Nothing

    End Function

    Public Function GetClientValidationRules(metadata As ModelMetadata, context As ControllerContext) As IEnumerable(Of ModelClientValidationRule) Implements IClientValidatable.GetClientValidationRules
        Dim rule = New ModelClientValidationRule()

        rule.ValidationType = "mtelvalidator"
        rule.ErrorMessage = ""
        rule.ValidationParameters("opts1") = Me.M_TEL1
        rule.ValidationParameters("opts2") = Me.M_TEL2
        rule.ValidationParameters("opts3") = Me.M_TEL3

        Dim list = New List(Of ModelClientValidationRule)()
        list.Add(rule)
        Return list
    End Function

End Class


<AttributeUsage(AttributeTargets.Property, AllowMultiple:=False)>
Public Class RequiredOccupationAttribute
    Inherits ValidationAttribute
    Implements IClientValidatable

    Public Overrides Function IsValid(ByVal value As Object) As Boolean

        If value = "0" Then
            Return False
        Else
            Return True
        End If

    End Function

    Public Function GetClientValidationRules(metadata As ModelMetadata, context As ControllerContext) As IEnumerable(Of ModelClientValidationRule) Implements IClientValidatable.GetClientValidationRules
        Dim rule = New ModelClientValidationRule()

        rule.ValidationType = "requiredoccupation"
        rule.ErrorMessage = FormatErrorMessage(metadata.GetDisplayName())
        rule.ValidationParameters("opts") = ""

        Dim list = New List(Of ModelClientValidationRule)()
        list.Add(rule)
        Return list
    End Function
End Class

<AttributeUsage(AttributeTargets.Property, AllowMultiple:=False)>
Public Class ExistsOperatorIDAttribute
    Inherits ValidationAttribute

    Public Overrides Function IsValid(ByVal value As Object) As Boolean

        If value = "" Then Return True

        Dim service As New UserService

        Dim ret As Boolean = service.ExistsOperatorID(value)

        Return Not ret

    End Function
End Class

<AttributeUsage(AttributeTargets.Property, AllowMultiple:=False)>
Public Class CorrectPasswordAttribute
    Inherits ValidationAttribute

    Public Property OPERATOR_ID As String
    Public Property PASSWORD As String

    Public Sub New(ByVal argOperatorId As String, ByVal argPassword As String)
        Me.OPERATOR_ID = argOperatorId
        Me.PASSWORD = argPassword
    End Sub



    Protected Overrides Function IsValid(ByVal value As Object, ByVal validationContext As ValidationContext) As ValidationResult
        Dim piOperatorId As PropertyInfo = validationContext.ObjectType.GetProperty(OPERATOR_ID)
        Dim piPassword As PropertyInfo = validationContext.ObjectType.GetProperty(PASSWORD)

        If piOperatorId.GetValue(validationContext.ObjectInstance, Nothing) = "" Then
            Return Nothing
        End If
        If piPassword.GetValue(validationContext.ObjectInstance, Nothing) = "" Then
            Return Nothing
        End If

        Dim service As New UserService

        Dim ret As Boolean = service.CorrectPassword(piOperatorId.GetValue(validationContext.ObjectInstance, Nothing), piPassword.GetValue(validationContext.ObjectInstance, Nothing))

        If ret = True Then
            Return Nothing
        Else
            Return New ValidationResult("")
        End If

    End Function
End Class

<AttributeUsage(AttributeTargets.Property, AllowMultiple:=False)>
Public Class SelectRoleAttribute
    Inherits ValidationAttribute

    Public Overrides Function IsValid(ByVal value As Object) As Boolean
        Dim b As Boolean() = DirectCast(value, Boolean())

        For i As Integer = 0 To b.Length - 1
            If b(i) = True Then
                Return True
            End If
        Next

        Return False
    End Function
End Class

<AttributeUsage(AttributeTargets.Property, AllowMultiple:=False)>
Public Class ValidatePasswordAttribute
    Inherits ValidationAttribute
    Implements IClientValidatable

    Public Overrides Function IsValid(ByVal value As Object) As Boolean

        If value Is Nothing Then Return True

        Dim pattern1 As Regex = New Regex("^[a-zA-Z0-9!""#$%&()*+-,./:;<=>?@\[\]^_`{\|}~\']{6,20}$")
        Dim pattern2 As Regex = New Regex("[0-9]")
        Dim pattern3 As Regex = New Regex("[a-z]")
        Dim pattern4 As Regex = New Regex("[A-Z]")
        If pattern1.IsMatch(value) Then
            If pattern2.IsMatch(value) And pattern3.IsMatch(value) And pattern4.IsMatch(value) Then
                Return True
            End If
        End If
        Return False
    End Function

    Public Function GetClientValidationRules(ByVal metadata As System.Web.Mvc.ModelMetadata, ByVal context As System.Web.Mvc.ControllerContext) As System.Collections.Generic.IEnumerable(Of System.Web.Mvc.ModelClientValidationRule) Implements System.Web.Mvc.IClientValidatable.GetClientValidationRules

        Dim rule = New ModelClientValidationRule()

        rule.ValidationType = "validatepassword"
        rule.ErrorMessage = FormatErrorMessage(metadata.GetDisplayName())
        rule.ValidationParameters("opts") = ""

        Dim list = New List(Of ModelClientValidationRule)()
        list.Add(rule)
        Return list
    End Function

End Class

<AttributeUsage(AttributeTargets.Property, AllowMultiple:=True)>
Public Class StartEndAttribute
    Inherits ValidationAttribute
    Implements IClientValidatable


    Public Property Value1 As String
    Public Property Value2 As String

    Public Sub New(ByVal argValue1 As String, ByVal argValue2 As String)
        Me.Value1 = argValue1
        Me.Value2 = argValue2
    End Sub

    Protected Overrides Function IsValid(ByVal value As Object, ByVal validationContext As ValidationContext) As ValidationResult
        Dim piValue1 As PropertyInfo = validationContext.ObjectType.GetProperty(Value1)
        Dim piValue2 As PropertyInfo = validationContext.ObjectType.GetProperty(Value2)

        If piValue1.GetValue(validationContext.ObjectInstance, Nothing) >= piValue2.GetValue(validationContext.ObjectInstance, Nothing) Then
            Return New ValidationResult("")
        Else
            Return Nothing
        End If
    End Function

    Public Function GetClientValidationRules(metadata As ModelMetadata, context As ControllerContext) As IEnumerable(Of ModelClientValidationRule) Implements IClientValidatable.GetClientValidationRules
        Dim rule = New ModelClientValidationRule()

        rule.ValidationType = "startend"
        rule.ErrorMessage = FormatErrorMessage(metadata.GetDisplayName())
        rule.ValidationParameters("opts1") = Me.Value1
        rule.ValidationParameters("opts2") = Me.Value2

        Dim list = New List(Of ModelClientValidationRule)()
        list.Add(rule)
        Return list
    End Function
End Class
